import ast
import pandas as pd
from kbodata.parser.util import make_primary_key, get_game_info


def pitcher_modify(data):
    """수집한 여러 개의 경기가 들어 있는 자료에서 투수 자료만 정리하는 함수

    이 함수는 여러 경기자료(`data`)에서 스코어보드만 뽑아서 내용을 고치고 변경한 다음
    다시 원 자료(`data`)에 끼워 넣는다. 즉 반환 값에는 모든 수집한 내용이 들어 있다.
    참고로 아래 긴 `for`문은 18회까지 연장하기 위한 방법이다. 기본적으로 현재 정규 이닝은
    13회까지밖에 없지만, 예전 정규 KBO 리그에서 18회까지 있는 경우가 있어 이를 반영했다.

    Args:
        data (json): 수집한 하나 이상의 경기 자료

    Returns:
        data (json): 투수 자료만 수정한 하나 이상의 경기 자료
    """
    i = 0
    home_or_away_list = ["away_pitcher", "home_pitcher"]
    game_info = get_game_info(data["id"])
    for home_or_away in home_or_away_list:
        pitchers = data['contents'][home_or_away]
        # 투수 자료의 경우 필요 없는 키들이 있어서 리스트를 새로 만들어서 덮어씌우기
        fin_pitchers=[]
        for pitcher in pitchers:
            new_info={}
            new_info['idx'] = make_primary_key(pitcher["팀"], game_info["year"], game_info["month"], game_info["day"], game_info["더블헤더"])
            new_info['name'] = pitcher["선수명"]
            new_info['team'] = pitcher["팀"]
            new_info['mound'] = '1' if pitcher['등판'] == '선발' else '0'
            new_info['inning'] = change_inning(pitcher['이닝'])
            new_info['result'] = change_result(pitcher['결과'])
            new_info['strikeout'] = pitcher['삼진']
            new_info['dead4ball'] = pitcher['4사구']
            new_info['losescore'] = pitcher['실점']
            new_info['earnedrun'] = pitcher['자책']
            new_info['pitchnum'] = pitcher['투구수']
            new_info['hitted'] = pitcher['피안타']
            new_info['homerun'] = pitcher['홈런']
            new_info['battednum'] = pitcher['타수']
            new_info['batternum'] = pitcher['타자']
            fin_pitchers.append(new_info)

        fin_pitchers= pd.DataFrame(fin_pitchers)
        data["contents"][home_or_away] = ast.literal_eval(fin_pitchers.to_json(orient="records"))
    i = i + 1

    return data


def change_inning(data):
    """이닝수와 나머지를 정리해주는 코드. 이닝수와 나머지 값을 순차적으로 나열한다. 이닝수의 경우 0-18까지 가능하며, 나머지의 경우 0-2까지만 가능하다.

    ### Examples:
        - '5' -> 50
        - '2\\/3' -> 02
    - '1 2\\/3' -> 12
    """
    temp = str(data).split()
    
    if len(temp) == 1 and "\\" not in temp[0]:
        return temp[0]+"0"
    elif len(temp) == 1 and "\\" in temp[0]:
        return "0"+temp[-1][0]
    else:
        return temp[0] + temp[-1][0]

def change_result(result):

    if result == '세이브':
        return "S"
    elif result == "홀드":
        return "H"
    elif result == "승":
        return "W"
    else:
        return "L"



import ast
import os
import re
import configparser
import pandas as pd
from kbodata.parser.util import make_primary_key, get_game_info

# 설정파일을 읽어오기 위해 configparser를 사용
config = configparser.ConfigParser()
# 필요한 변수 가져오기
config.read(os.path.join(os.path.dirname(__file__),"config.ini"), encoding="utf-8")
Batter_factor = config["BATTER"]


def batter_modify(data):
    """수집한 여러 개의 경기가 들어 있는 자료에서 타자 자료만 정리하는 함수

    이 함수는 여러 경기자료(`data`)에서 스코어보드만 뽑아서 내용을 고치고 변경한 다음
    다시 원 자료(`data`)에 끼워 넣는다. 즉 반환 값에는 모든 수집한 내용이 들어 있다.
    참고로 아래 긴 `for`문은 18회까지 연장하기 위한 방법이다. 기본적으로 현재 정규 이닝은
    13회까지밖에 없지만, 예전 정규 KBO 리그에서 18회까지 있는 경우가 있어 이를 반영했다.

    Args:
        data (json): 수집한 하나 이상의 경기 자료

    Returns:
        data (json): 타자 자료만 수정한 하나 이상의 경기 자료
    """
    i = 0

    home_or_away_list = ["away_batter", "home_batter"]
    game_info = get_game_info(data["id"])
    for home_or_away in home_or_away_list:
        batters = data["contents"][home_or_away]
        # 타자 자료의 경우 필요 없는 키들이 있어서 리스트를 새로 만들어서 덮어씌우기
        fin_batters = []
        for batter in batters:
            new_info = {}
            new_info["idx"] = make_primary_key(
                batter["팀"],
                game_info["year"],
                game_info["month"],
                game_info["day"],
                game_info["더블헤더"],
            )
            new_info["name"] = change_long_name(batter["선수명"])
            new_info["team"] = batter["팀"]
            new_info["position"] = change_position(batter["포지션"])
            new_info = add_ining(Batter_factor, new_info, batter)
            new_info["hit"] = batter["안타"]
            new_info["bat_num"] = batter["타수"]
            new_info["hit_get"] = batter["타점"]
            new_info["own_get"] = batter["득점"]
            fin_batters.append(new_info)

        fin_batters = pd.DataFrame(fin_batters)
        data["contents"][home_or_away] = ast.literal_eval(
            fin_batters.to_json(orient="records")
        )
    i = i + 1

    return data


def change_position(data):
    """
    data = pandas DF
    사용방법
    import pandas as pd
    temp = pd.read_json("20210409_KTSS0.json")
    batter = pd.DataFrame(temp['20210409_KTSS0']["away_batter"])
    change_posision(batter)
    """
    pst = re.split("\B", data)

    for _ in pst:
        if "一" in data:
            data = data.replace("一", "3")
        elif "二" in data:
            data = data.replace("二", "4")
        elif "三" in data:
            data = data.replace("三", "5")
        elif "투" in data:
            data = data.replace("투", "1")
        elif "포" in data:
            data = data.replace("포", "2")
        elif "유" in data:
            data = data.replace("유", "6")
        elif "좌" in data:
            data = data.replace("좌", "7")
        elif "중" in data:
            data = data.replace("중", "8")
        elif "우" in data:
            data = data.replace("우", "9")
        elif "지" in data:
            data = data.replace("지", "D")
        elif "주" in data:
            data = data.replace("주", "R")
        elif "타" in data:
            data = data.replace("타", "H")
    return data


def add_ining(config, new_data, data):
    """
    이닝 수를 최대값인 18에 맞춰서 추가해주고 키 이름도 변경해주는 함수
    """

    for i in range(1, 19):
        if str(i) in data:
            # 키 이름 변경
            new_data["i_" + str(i)] = trans_code(config, str(data.pop(str(i))))
        else:
            # 데이터 추가
            new_data["i_" + str(i)] = "-"

    return new_data


def trans_code(config, data):
    """붙어있는 이닝 결과값들을 변환해주는 코드
    Args:
        data (sting): 한글로 기록된 타격기록
    Returns:
        data (int): "code_list.ini"로 변환된 코드
    """
    temp = [config[change_record(x)] for x in re.split("\W", data) if x != ""]

    return "".join(temp)


def change_record(data):
    """2010년 이전데이터에서 존재하는 타격기록의 한자를 숫자로 변경하는 함수
  
    ### Args:
        data (str): 한자가 포함된 타격기록

    ### Returns:
        temp_data (str): 숫자로 변경된 타격기록
    """
    data = data.replace("一","1")
    data = data.replace("二","2")
    data = data.replace("三","3")
    return data

def change_long_name(name):
    
    full_names = {"페르난데":"페르난데스","해즐베이":"해즐베이커","스몰린스":"스몰린스키","반슬라이":"반슬라이크"}
    if name in full_names.keys():
        return  full_names[name]
    else:
        return name



import pandas as pd

def batter_output(data):
    
    result = []
    batter = [item["contents"]["away_batter"] for item in data]
    batter += [item["contents"]["home_batter"] for item in data]
    for item in batter:
        result += item
    
    return result

def batter_to_DataFrame(data):
    
    result = batter_output(data)
    result = pd.DataFrame(result)
    return result

def batter_to_Dict(data):
    
    result = batter_output(data)
    return result



"""KBO 정규 시즌 게임 자료를 가져와서 사용할 수 있도록 가공하기 쉽게 수정해주는 모듈 
"""
from selenium import webdriver
from kbodata.parser.batter import batter_modify
from tqdm import tqdm
from kbodata.parser.page import parsing_single_game
from kbodata.parser.scoreboard import scoreboard_modify
from kbodata.parser.batter import batter_modify
from kbodata.parser.pitcher import pitcher_modify

def get_game_data(schedule, Driver_path):
    """스케쥴에 해당하는 데이터를 가져오는 함수
    데이터들을 스크래핑하여 list로 반환한다.
    """
    data = []
    options = webdriver.ChromeOptions()
    options.add_argument("headless")
    options.add_argument("window-size=1920x1080")
    options.add_argument("disable-gpu")
    options.add_argument("--log-level=2")
    # 혹은 options.add_argument("--disable-gpu")
    driver = webdriver.Chrome(Driver_path, options=options)
    driver.implicitly_wait(100)

    with tqdm(desc="in progress",total=len(schedule)) as pbar:
        for idx, row in schedule.iterrows():
            if row["status"] == 'canceled':
                pbar.update(1)
            else:
                raw_data = parsing_single_game(row["date"],row["gameid"],driver)
                scoreboard_modify(raw_data)
                batter_modify(raw_data)
                pitcher_modify(raw_data)
                data.append(raw_data)
                pbar.update(1)
    driver.quit()
    return data



import os
import configparser
import pandas as pd
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from kbodata.parser.util import change_name_to_id

# 설정파일을 읽어오기 위해 configparser를 사용
config = configparser.ConfigParser()
# 필요한 변수 가져오기
config.read(os.path.join(os.path.dirname(__file__),"config.ini"), encoding="utf-8")
info_url = config["DEFAULT"]["Game_info_URL"]


def parsing_monthly_schedule(year, month, driver):

    driver.get(info_url)
    driver.find_element_by_id("ddlYear").send_keys(str(year))
    driver.find_element_by_id("ddlMonth").send_keys(str(month).zfill(2))
    data = WebDriverWait(driver, 100).until(EC.visibility_of_element_located((By.ID,"tblSchedule")))
    # 스크래핑 된 데이터 정리
    table = BeautifulSoup(data.get_attribute('innerHTML'), "lxml")
    result = []
    not_listed = ["EAST", "WEST", "드림", "나눔"]
    for td in table.find_all("td"):
        # 경기 없는 날 확인
        if len(td) == 1:
            continue
        for li in td.find_all("li"):
            info = (li.text).split()
            # 경기날짜 확인
            if li.get('class') == ['dayNum']:
                day = str(year)+str(month).zfill(2) + info[0].zfill(2)
            # 나눔 경기는 제외
            elif info[0] in not_listed:
                continue
            # 경기 취소 확인
            elif (li.get('class') == ['rainCancel']) or (day == "20111025"):
                status = "canceled"
                home = change_name_to_id(info[2],year)
                away = change_name_to_id(info[0],year)
                result.append([status,day,home,away])
            else:
                status = "finished"
                home = change_name_to_id(info[-1],year)
                away = change_name_to_id(info[0],year)
                result.append([status,day,home,away])
    result = pd.DataFrame(result, columns=["status","date","home","away"])
    result = add_gameid(result)
    result = delete_non_provided_data(result)
    
    return result


def parsing_daily_schedule(year,month,day,driver):

    # 스케쥴 데이터 스크래핑
    driver.get(info_url)
    driver.find_element_by_id("ddlYear").send_keys(str(year))
    driver.find_element_by_id("ddlMonth").send_keys(str(month).zfill(2))
    data = WebDriverWait(driver, 100).until(EC.visibility_of_element_located((By.ID,"tblSchedule")))
    # 스크래핑 된 데이터 정리
    table= BeautifulSoup(data.get_attribute('innerHTML'), "lxml")
    result = []
    for td in table.find_all("td"):
        if td.find("li",{"class":"dayNum"}).text != str(day):
            continue
        for li in td.find_all("li"):
            info = (li.text).split()
            # 경기날짜 확인
            if li.get('class') == ['dayNum']:
                day = str(year)+str(month).zfill(2) + info[0].zfill(2)
            # 경기 취소 확인
            elif li.get('class') == ['rainCancel']:
                status = "canceled"
                home = change_name_to_id(info[2],year)
                away = change_name_to_id(info[0],year)
                result.append([status,day,home,away])
            else:
                status = "finished"
                home = change_name_to_id(info[-1],year)
                away = change_name_to_id(info[0],year)
                result.append([status,day,home,away])
    result = pd.DataFrame(result, columns=["status","date","home","away"])
    result = add_gameid(result)
    result = delete_non_provided_data(result)

    return result


def add_gameid(result):
    """gameid를 생성한다.  gameid는 (away+home+dbheader)로 구성된 문자열이다.
       더블헤더를 확인하는 기준은 다음과 같다.  더블헤더 x: 0 / 더블헤더 o: 1(첫번째 경기), 2(두번째 경기)
    """
    # 더블헤더 여부 저장할 열 생성
    result["dbheader"] = 0
    
    # 날짜 별 경기 횟수 조회
    temp = result.groupby(["date","away","home"],as_index=False).count()
    
    # 그 중 더블헤더 경기만 추출
    checker = temp.loc[temp["status"] == 2]
    
    # 더블헤더 중 첫 경기가 취소된 경기 제거
    dbheader = checker.copy()
    for idx, dbhd in dbheader.iterrows():
        bh = dbhd["date"]+dbhd["away"]+dbhd["home"]
        stat = list(result.loc[result["date"]+result["away"]+result["home"] == bh]["status"])
        if (stat[0] == 'canceled') & (stat[1] == 'finished'):
                dbheader = dbheader.drop(idx)
    
    # 더블헤더 경기인 경우 1, 2 로 입력
    for idx, dbhd in dbheader.iterrows():
        bh = dbhd["date"]+dbhd["away"]+dbhd["home"]
        count = 1
        for jdx, data in result.iterrows():
            dt = data["date"]+data["away"]+data["home"] 
            if dt == bh:
                result.iat[jdx, 4] = count
                count += 1
    # 더블헤더와 팀 정보로 gameid 생성
    result["gameid"] = result[["away","home","dbheader"]].apply(lambda row: ''.join(row.values.astype(str)), axis=1)

    return result

def delete_non_provided_data(result):
    """경기정보가 제공되지 않은 스케쥴을 제거하는 함수
    """
    not_provided = [("20080330","LTHH0"),
                    ("20090404","WOLT0"),
                    ("20100320","OBLT0"),
                    ("20100320","WOSS0"),
                    ("20150708","HTWO0"),
                    ("20180801","WOSK0")]
    for day, gameid in not_provided:
        idx = result[(result["date"]==day)&(result["gameid"]==gameid)].index
        result = result.drop(idx)
        result.reset_index(inplace=True,drop=True)

    not_provided_for_day = ["20130309",
                            "20130310",
                            "20130311",
                            "20130312",
                            "20130313",
                            "20130314",
                            "20130315",
                            "20130316",
                            "20130317",
                            "20130318",
                            "20130319",
                            "20130320",
                            ]
    for day in not_provided_for_day:
        idx = result[(result["date"]==day)].index
        result = result.drop(idx)
        result.reset_index(inplace=True,drop=True)

    return result



from datetime import date
from tqdm import tqdm
from dateutil.relativedelta import relativedelta
from selenium import webdriver
import pandas as pd
from kbodata.parser.schedule import parsing_daily_schedule, parsing_monthly_schedule

def get_daily_schedule(year,month,day,Driver_path):
    """유저 함수: 2008년 이후 데이터 중에 요청된 날짜의 경기 스케쥴을 가져온다.

    ex) get_daily_schedule("20210420","chromedriver")

        status      date home away  dbheader gameid
    0	finished	20210420	HT	LG	0	HTLG0
    1	finished	20210420	KT	NC	0	KTNC0
    2	finished	20210420	OB	LT	0	OBLT0
    3	finished	20210420	SK	SS	0	SKSS0
    4	finished	20210420	WO	HH	0	WOHH0
    """

    if (str(year)+str(month)+str(day)).isdigit() == False:
        return print("ERROR: INVALID PARAMETER. please check year or month or day")

    sd_date = date(year, month, day)
    options = webdriver.ChromeOptions()
    options.add_argument("headless")
    options.add_argument("window-size=1920x1080")
    options.add_argument("disable-gpu")
    options.add_argument("--log-level=2")
    driver = webdriver.Chrome(Driver_path, options=options)
    driver.implicitly_wait(100)

    if sd_date.year < 2008: return print("ERROR: This library only provides data since 2008.")
    if sd_date > date.today(): return print("ERROR: The end date is later than now.")
    
    data = parsing_daily_schedule(year,month,day,driver)
    driver.quit()
    return data

def get_monthly_schedule(year, month, Driver_path):
    """유저 함수: 2008년부터 달 단위로 요청된 경기 스케쥴을 가져온다.
    """
    if (str(year)+str(month)).isdigit() == False:
        return print("ERROR: INVALID PARAMETER. please check year or month")
    st_date = date(year,month,1)
    
    if st_date.year < 2008: return print("ERROR: This library only provides data since 2008.")
    if st_date > date.today(): return print("ERROR: The date is later than now.")

    options = webdriver.ChromeOptions()
    options.add_argument("headless")
    options.add_argument("window-size=1920x1080")
    options.add_argument("disable-gpu")
    driver = webdriver.Chrome(Driver_path, options=options)
    driver.implicitly_wait(100)

    data = parsing_monthly_schedule(st_date.year,st_date.month,driver)
    driver.quit()

    return data

def get_yearly_schedule(year, Driver_path):
    """유저 함수: 2008년부터 년 단위로 요청된 경기 스케쥴을 가져온다.
    """
    if len(str(year)) != 4 or (str(year)).isdigit() == False:
        return print("ERROR: INVALID PARAMETER. please check year")
    schedule = pd.DataFrame()
    st_date = date(year,1,1)
    ed_date = date(year,12,30)
    
    if st_date.year < 2008: return print("ERROR: This library only provides data since 2008.")
    if ed_date.year > date.today().year : return print("ERROR: The end date is later than now.")
    
    delta = relativedelta(ed_date, st_date)
    options = webdriver.ChromeOptions()
    options.add_argument("headless")
    options.add_argument("window-size=1920x1080")
    options.add_argument("disable-gpu")
    driver = webdriver.Chrome(Driver_path, options=options)
    driver.implicitly_wait(100)
    
    with tqdm(desc="in progress",total=12) as pbar:
        while st_date < ed_date:
            data = parsing_monthly_schedule(st_date.year,st_date.month,driver)
            schedule = pd.concat([schedule,data],axis=0,ignore_index=True)
            st_date += relativedelta(months=1)
            pbar.update(1)

    driver.quit()
    return schedule



import ast
import os
import configparser
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from kbodata.parser.html import scoreboard, etc_info, looking_for_team_names
from kbodata.parser.html import away_batter, home_batter, away_pitcher, home_pitcher

# 설정파일을 읽어오기 위해 configparser를 사용
config = configparser.ConfigParser()
# 필요한 변수 가져오기
config.read(os.path.join(os.path.dirname(__file__),"config.ini"), encoding="utf-8")
url = config["DEFAULT"]["KBO_URL"]


def parsing_page(gameDate, gameId, driver):
    temp_url = url + gameDate + "&amp;gameId=" + gameDate + gameId + "&amp;section=REVIEW"
    driver.get(temp_url)
    data = WebDriverWait(driver, 100).until(EC.visibility_of_element_located((By.ID,"gameCenterContents")))
    soup = BeautifulSoup(data.get_attribute('innerHTML'), "lxml")
    tables = soup.find_all("table")
    record_etc = soup.findAll("div", {"class": "record-etc"})
    temp_teams = soup.findAll("h6")
    teams = looking_for_team_names(temp_teams)

    return {
        "tables": tables,
        "record_etc": record_etc,
        "teams": teams,
        "date": gameDate,
        "id": gameId,
    }


def parsing_single_game(date, gameId, driver):
    """
    다운받은 단일 게임 자료를 사용하기 쉽게 원본을 보존하며 적절하게 정리하는 함수

    `get_page()`을 통해 받은 단일 게임 자료를
    데이터 분석하기 위해 modify 하기 쉽도록 다운받은 내용 그대로 유지하면서
    내용을 추가하지 않고 필요하지 않은 부분, 예를 들어 HTML 관련 코드 등을 정리한다.
    이렇게 처리하는 이유는 다운받은 것을 거의 원본 그대로 보관하면
    이 자료를 다룰 때 문제점이 발생하더라도 다운을 다시 받을 필요가 없기 때문이다.

    Example
    -------
    >>> temp_page = parsing_single_game("20181010","KTLT1", driver)

    Parameters
    ----------
    :param1: str
        date: "20181010" 와 같이 경기 날짜로 만들어진 문자열
    :param1: str
        gameld: 경기를 하는 팀명과 더블해더 유무를 이용해 만든 문자열
        "WOOB0"과 같이 만드는데, WO, OB는 각각 팀명을 의미하고
        0은 더블헤더 경기가 아닌 것을 알려준다.
        만약 더불헤더 경기면 1차전은 "KTLT1"처럼 1로 표시하고
        2차전이면 "KTLT2"으로 표시한다.

    Returns
    -------
        json
        게임 자료가 들어 있다. "id" 키에는 '20181010_KTLT1' 과 같은
        정규 시즌 단일 게임 아이디가 들어 있다. Parameters를 이용해서 만든다.
        "contents" 키에는 다음 키가 들어 있고 각 키에는 해당 게임 정보가 들어 있다.

        - 'scoreboard'
        - 'ETC_info'
        - 'away_batter'
        - 'home_batter'
        - 'away_pitcher'
        - 'home_pitcher'
    """

    temp_page = parsing_page(date, gameId, driver)

    temp_scoreboard = scoreboard(temp_page["tables"], temp_page["teams"])

    temp_all = {
        "scoreboard": ast.literal_eval(temp_scoreboard.to_json(orient="records"))
    }
    temp_all.update(
        {"ETC_info": etc_info(temp_page["tables"], temp_page["record_etc"])}
    )
    temp_all.update(
        {
            "away_batter": ast.literal_eval(
                away_batter(temp_page["tables"], temp_page["teams"]).to_json(
                    orient="records"
                )
            )
        }
    )
    temp_all.update(
        {
            "home_batter": ast.literal_eval(
                home_batter(temp_page["tables"], temp_page["teams"]).to_json(
                    orient="records"
                )
            )
        }
    )
    temp_all.update(
        {
            "away_pitcher": ast.literal_eval(
                away_pitcher(temp_page["tables"], temp_page["teams"]).to_json(
                    orient="records"
                )
            )
        }
    )
    temp_all.update(
        {
            "home_pitcher": ast.literal_eval(
                home_pitcher(temp_page["tables"], temp_page["teams"]).to_json(
                    orient="records"
                )
            )
        }
    )

    temp_name = temp_page["date"] + "_" + temp_page["id"]
    return {"id": temp_name, "contents": temp_all}



import datetime
import configparser
import os

# 설정파일을 읽어오기 위해 configparser를 사용
config = configparser.ConfigParser()
# 필요한 변수 가져오기
config.read(os.path.join(os.path.dirname(__file__),"config.ini"), encoding="utf-8")

def change_name_to_id(team_name, year):
    """팀 이름을 팀 ID로 바꾸는 함수
    2021년 SSG가 창단했다. 그래서 팀명이 SK에서 SSG로 변경되었다.
    그러나 KBO 홈피에서 사용하는 ID는 안 바꾼 것 같다.
    그래서 팀명을 KBO에서 바꾸는 함수를 새로 만들었다.
    Args:
        team_name (str): 팀명
    """

    team_list_2021 = {
        "KIA": "HT",
        "두산": "OB",
        "롯데": "LT",
        "NC": "NC",
        "SSG": "SK",
        "LG": "LG",
        "넥센": "WO",
        "키움": "WO",
        "히어로즈": "WO",
        "우리": "WO",
        "한화": "HH",
        "삼성": "SS",
        "KT": "KT",
    }
    team_list = {
        "KIA": "HT",
        "두산": "OB",
        "롯데": "LT",
        "NC": "NC",
        "SK": "SK",
        "LG": "LG",
        "넥센": "WO",
        "키움": "WO",
        "히어로즈": "WO",
        "우리": "WO",
        "한화": "HH",
        "삼성": "SS",
        "KT": "KT"
    }

    if year >= 2021:
        return team_list_2021[team_name]
    else:
        return team_list[team_name]

def get_game_info(game_list):
    """입력된 정보를 토대로 해당 경기 연도, 날짜, 요일 등을 만드는 함수

    `20211115001`과 같은 정보가 들어오면 이를 가지고
    해당 경기 연도, 날짜, 요일 등을 만든다.

    Args:
        game_list (str): `20211115001`

    Returns:
        (dict): "year", "month", "day", "week", "더블헤더"를 키로 포함한다.
    """

    temp_date = game_list.split("_")[0]
    temp_date = datetime.datetime.strptime(temp_date.split("_")[0], "%Y%m%d")
    temp = {
        "year": temp_date.year,
        "month": temp_date.month,
        "day": temp_date.day,
        "week": temp_date.weekday(),
    }
    temp_team = game_list.split("_")[1]
    temp_team = {
        "더블헤더": int(temp_team[4:]),
    }
    temp.update(temp_team)

    return temp


def make_primary_key(team_name, year, month, day, dbheader):
    """스코어보드 DB에서 사용할 Primary Key를 작성하는 함수

    Examples:

        ```python
        year = 2021
        month = 4
        day = 29
        team_name = '두산'
        dbheader = 0

        import scoreboards
        scoreboards.make_primary_key(team_name, year, month, day, dbheader)
        '20210429001'
        ```

    Args:
        year (int):
        month (int) :
        day (int) :
        team_name (str) : 팀명 EG: 두산
        dbheader (int) : 더블해더 경기 유무.  아니다: 0, 1차전: 1, 2차전: 2

    Returns:
        (str): 숫자 길이가 11인 자연수. E.G.: '20210429001'
    """
    result = (
        str(year)
        + str(month).zfill(2)
        + str(day).zfill(2)
        + str(dbheader)
        + change_id_to_number(team_name).zfill(2)
    )
    return result


def change_id_to_number(team_name):
    """팀명을 팀 TeamID로 바꾸는 함수

    만약 빈 팀명이 들어오면, 히어로스 TeamID이 반환됩니다.
    2008~2009까지 스폰서가 없어서 "서울 히어로즈"라고 했지만,
    공식적으로 스폰서가 없었기 때문에 KBO에서는 "히어로즈"라고 명명하고 있다.
    그래서 빈 팀명이 들어오게 된다.

    Examples:

        ```python
        >>> import modifying
        >>> modifying.changing_team_name_into_id("두산")
        >>> '1'
        >>> modifying.changing_team_name_into_id("")
        >>> '8'
        ```

    Args:
        team_name (str): 팀명

    Returns:
        (str): 자연수 숫자
    """

    if team_name == "":
        return "8"
    else:
        return config["TEAM"][team_name]



import pandas as pd

def scoreboard_output(data):
    
    scoreboard = [item["contents"]["scoreboard"] for item in data]
    result = []
    for item in scoreboard:
        result.append(item[0])
        result.append(item[1])
    
    return result

def scoreboard_to_DataFrame(data):
    
    result = scoreboard_output(data)
    result = pd.DataFrame(result)
    return result

def scoreboard_to_Dict(data):

    result = scoreboard_output(data)
    return result



import pandas as pd

def pitcher_output(data):
    
    result = []
    pitcher = [item["contents"]["away_pitcher"] for item in data]
    pitcher += [item["contents"]["home_pitcher"] for item in data]
    for item in pitcher:
        result += item
    
    return result

def pitcher_to_DataFrame(data):

    result = pitcher_output(data)
    result = pd.DataFrame(result)
    return result

def pitcher_to_Dict(data):
    
    result = pitcher_output(data)
    return result



__version__ = '0.1.4'

from kbodata.get.schedule import(
    get_daily_schedule,
     get_monthly_schedule,
     get_yearly_schedule
     )

from kbodata.get.game import get_game_data
from kbodata.load.scoreboard import scoreboard_to_DataFrame, scoreboard_to_Dict
from kbodata.load.batter import batter_to_DataFrame, batter_to_Dict
from kbodata.load.pitcher import pitcher_to_DataFrame, pitcher_to_Dict



import ast
import pandas as pd

from kbodata.parser.util import make_primary_key, get_game_info

def scoreboard_modify(data):
    """수집한 여러개의 경기가 들어 있는 자료에서 스코어보드만 정리하는 함수
    이 함수는 여러 경기자료(`data`)에서 스코어보드만 뽑아서 내용을 고치고 변경한 다음
    다시 원 자료(`data`)에 끼워 넣는다. 즉 반환 값에는 모든 수집한 내용이 들어 있다.
    참고로 아래 긴 `for`문은 18회까지 연장하기 위한 방법이다. 기본적으로 현재 정규 이닝은
    13회까지밖에 없지만, 예전 정규 KBO 리그에서 18회까지 있는 경우가 있어 이를 반영했다.
    Note:
        현재 수정하고 있는 컬럼 이름
        - 이닝 이름 12개
        - 승패
        - 홈팀
        - 원정팀
        - 더블헤더
    Args:
        data (json): 수집한 하나 이상의 경기 자료
    Returns:
        data (json): scoreboard만 수정한, 하나 이상의 경기 자료
    """
    i = 0
    fin_boards = []
    etc_info = data["contents"]["ETC_info"]
    game_info = get_game_info(data["id"])
    home = data["contents"]["scoreboard"][1]["팀"]
    away = data["contents"]["scoreboard"][0]["팀"]
    for old_info in data["contents"]["scoreboard"]:
        new_info = {}
        new_info["idx"] = make_primary_key(
            old_info["팀"],
            game_info["year"],
            game_info["month"],
            game_info["day"],
            game_info["더블헤더"],
            )
        new_info["team"] = old_info["팀"]
        new_info["result"] = 1 if old_info["승패"] == "승" else 0 if old_info["승패"] == "무" else -1
        new_info = add_ining(new_info, old_info)
        new_info["r"] = old_info['R']
        new_info["h"] = old_info['H']
        new_info["e"] = old_info['E']
        new_info["b"] = old_info['B']
        new_info["year"] = game_info["year"]
        new_info["month"] = game_info["month"]
        new_info["day"] = game_info["day"]
        new_info["week"] = game_info["week"]
        new_info["home"]= home
        new_info["away"] = away
        new_info["dbheader"] = game_info["더블헤더"]
        new_info["place"] = etc_info["구장"]
        new_info["audience"] = int(etc_info["관중"].replace(',',''))
        new_info["starttime"] =  etc_info["개시"]
        new_info["endtime"] =  etc_info["종료"]
        new_info["gametime"] =  etc_info["경기시간"]
        fin_boards.append(new_info)

    fin_boards = pd.DataFrame(fin_boards)
    data["contents"]["scoreboard"] = ast.literal_eval(
            fin_boards.to_json(orient="records")
    )
    i = i + 1

    return data

def add_ining(new_data, data):
    """이닝 수를 최대값인 18에 맞춰서 추가해주고 키 이름도 변경해주는 함수
    """

    for i in range(1, 19):
        if str(i) in data:
            new_data["i_"+str(i)] = data[str(i)]
        else:
            # 데이터 추가
            new_data["i_" + str(i)] = "-"

    return new_data



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kbodata', 'kbodata.get', 'kbodata.load', 'kbodata.parser']

package_data = \
{'': ['*']}

install_requires = \
['bs4>=0.0.1,<0.0.2',
 'lxml>=4.6.2,<5.0.0',
 'pandas>=1.2.2,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'selenium>=3.141.0,<4.0.0']

setup_kwargs = {
    'name': 'kbodata',
    'version': '0.1.4',
    'description': 'Scraping Korea Baseball Game information',
    'long_description': "# What is kbo-data\n\nkbo-data는 한국프로야구 경기정보를 스크래핑하는 파이썬 패키지입니다.  \nkbo-data is a Python package that provides Korean professional baseball game information by scraping.\n\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/kbodata)\n[![PyPI](https://img.shields.io/pypi/v/kbodata)](https://pypi.org/project/kbodata/)\n[![GitHub license](https://img.shields.io/github/license/Hyeonji-Ryu/kbo-data)](https://github.com/Hyeonji-Ryu/kbo-data/blob/main/LICENSE)\n\n## Required\n\n이 패키지를 사용하기 위해서는 chrome driver가 필요합니다. chrome driver는 [해당 페이지](https://chromedriver.chromium.org/downloads)에서 다운로드할 수 있습니다.  \nThis package is required chrome driver. You can download it from [this page](https://chromedriver.chromium.org/downloads)\n\n## How to Use\n\n### 패키지 설치하기\n\n먼저 패키지를 설치합니다.  \nyou have to install kbodata package first.\n\n```bash\npip install kbodata\n```\n\n### 데이터 가져오기 (kbodata.get module)\n\n원하는 날짜의 경기 스케쥴을 다운로드 받습니다.  \nyou can download KBO match schedule that you want to get.\n\n```python\n    import kbodata\n\n    # 2021년 4월 20일의 KBO 경기 스케쥴을 가져옵니다.\n    # Get the KBO match schedule for April 20, 2021.\n    >>> day = kbodata.get_daily_schedule(2021,4,20,'chromedriver_path')\n\n    # 2021년 4월 KBO 경기 스케쥴을 가져옵니다.\n    # Get the KBO match schedule for April 2021.\n    >>> month = kbodata.get_monthly_schedule(2021,4,'chromedriver_path')\n\n    # 2021년 KBO 경기 스케쥴을 가져옵니다. \n    # Get the KBO match schedule for 2021.\n    >>> year = kbodata.get_yearly_schedule(2021,'chromedriver_path')\n```\n\n해당 스케쥴을 바탕으로 경기 정보를 JSON 형식으로 가져옵니다.  \nIt will be broght match information in JSON format based on the schedule.  \n\n```python\n    # 2021년 4월 20일의 KBO 경기 정보를 가져옵니다.\n    # Get the KBO match information for April 20, 2021.\n    >>> day_data = kbodata.get_game_data(day,'chromedriver_path')\n\n    # 2021년 4월 KBO 경기 정보를 가져옵니다.\n    # Get the KBO match information for April 2021.\n    >>> month_data = kbodata.get_game_data(month,'chromedriver_path')\n\n    # 2021년 KBO 경기 정보를 가져옵니다. \n    # Get the KBO match information for 2021.\n    >>> year_data = kbodata.get_game_data(year,'chromedriver_path')\n```\n\nJSON 형식은 아래와 같습니다.  \nThe JSON format is as below.\n\n```ini\n    { id: date_gameid,\n    contents: {\n      'scoreboard': []\n      'ETC_info': {}\n      'away_batter': []\n      'home_batter': []\n      'away_pitcher': []\n      'home_pitcher': []\n        }\n    }\n```\n\n## 데이터 변형하기 (kbodata.load module)\n\n가져온 데이터들을 특정 파일 타입으로 변환합니다. 지원하는 파일 타입은 아래와 같습니다.  \nThis module converts data into specific file types. The supported file types are as follows.\n\n- DataFrame(pandas)\n- Dict\n\n```python\n    # 팀 경기 정보만을 정리하여 DataFrame으로 변환합니다.\n    scoreboard = kbodata.scoreboard_to_DataFrame(day_data)\n    # 타자 정보만을 정리하여 DataFrame으로 변환합니다.\n    batter = kbodata.batter_to_DataFrame(day_data)\n    # 투수 정보만을 정리하여 DataFrame으로 변환합니다.\n    pitcher = kbodata.pitcher_to_DataFrame(day_data)\n\n    # 팀 경기 정보만을 정리하여 Dict으로 변환합니다.\n    scoreboard = kbodata.scoreboard_to_Dict(day_data)\n    # 타자 정보만을 정리하여 Dict으로 변환합니다.\n    batter = kbodata.batter_to_Dict(day_data)\n    # 투수 정보만을 정리하여 Dict으로 변환합니다.\n    pitcher = kbodata.pitcher_to_Dict(day_data)\n```\n\n변환된 데이터에 대한 정보는 아래의 링크에서 확인할 수 있습니다.  \nYou can find information about the converted data at the link below.\n\n- Scoreboard: https://github.com/Hyeonji-Ryu/kbo-data/blob/main/docs/scoreboard.md\n- Batter: https://github.com/Hyeonji-Ryu/kbo-data/blob/main/docs/batter.md\n- Pitcher: https://github.com/Hyeonji-Ryu/kbo-data/blob/main/docs/pitcher.md\n\n## Issues\n\nKBO 공식 홈페이지에 없는 데이터는 제공되지 않습니다. 데이터가 제공되지 않는 경기 정보는 아래와 같습니다.  \nData that is not on the KBO official website is not provided. Match information for which data is not provided are listed below.  \n\n### 경기 기준 (from game)\n\n- 2008-03-30 LTHH0\n- 2009-04-04 WOLT0\n- 2010-03-20 OBLT0\n- 2010-03-20 WOSS0\n- 2015-07-08 HTWO0\n- 2018-08-01 WOSK0\n\n### 날짜 기준 (from date)\n\n- 2013-03-09\n- 2013-03-10\n- 2013-03-11\n- 2013-03-12\n- 2013-03-13\n- 2013-03-14\n- 2013-03-15\n- 2013-03-16\n- 2013-03-17\n- 2013-03-18\n- 2013-03-19\n- 2013-03-20\n",
    'author': 'Hyeonji-Ryu',
    'author_email': 'guswl3940@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Hyeonji-Ryu/kbo-data',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)



import pandas as pd


def looking_for_team_names(temp_teams):
    """ 모은 HTML soup에서 시합한 두 팀명을 뽑는 함수
    다음과 같은 HTML soup에서 "한화"만 뽑아서 팀명을 찾는다.
    실제로는 아래와 같은 것이 두 팀에서 각 각 타자, 투수 총 네 번 나온다.
    거기서 두 팀만 뽑으면 된다. 또한 아래처럼 "한화 이글스 타자 기록"에서 첫 번째 단어만 뽑으면 된다.
    <h6 class="tit-team" id="lblAwayHitter">
        <span class="logo">
            <img src="//lgcxydabfbch3774324.cdn.ntruss.com \ 
            /KBO_IMAGE/emblem/regular/2020/initial_HH_s.png" alt="한화 이글스">
        </span>
    한화 이글스 타자 기록</h6>
        Args:
            temp_teams (soup): 위에서 보여준 html soup
        Returns:
            temp_data (tuple): (두산, LG)처럼 `str`형식으로 두 팀 명,
                                첫 번째 팀이 원정팀(두산), 두 번째 팀(LG)은 홈팀
    """

    temp_team_list = []

    for team in temp_teams:
        temp_team = team.get_text()
        temp_team = temp_team.split(" ")
        if len(temp_team[0]) == 0:
            team_name = temp_team[1]
        else:
            team_name = temp_team[0]
        if team_name not in temp_team_list:
            temp_team_list.append(team_name)

    return (temp_team_list[0], temp_team_list[1])

def scoreboard(tables, teams):
    temp_df_0 = pd.read_html(str(tables[0]))[0]
    temp_df_0 = temp_df_0.rename(columns={"Unnamed: 0": "승패"})
    temp_df_1 = pd.read_html(str(tables[1]))[0]
    temp_df_2 = pd.read_html(str(tables[2]))[0]
    temp_teams_df = pd.DataFrame({"팀": teams})
    temp_total = pd.concat(
        [temp_teams_df, temp_df_0["승패"], temp_df_1, temp_df_2], axis=1
    )
    return temp_total

def etc_info(tables, record_etc):
    record = {}
    header_list = tables[3].find_all("th")
    if len(header_list) != 0:
        header = [h.get_text(strip=True) for h in header_list]
        data = tables[3].find_all("td")
        etc_data = [d.get_text(strip=True) for d in data]
        record = {header[i]: etc_data[i] for i in range(0, len(header))}
        record.update(
            {
                key: record[key].split(") ")
                for key in record.keys()
                if len(record[key].split(") ")) >= 2
            }
        )
        record["심판"] = record["심판"].split(" ")
    etc = {
        record_etc[0]
        .find_all("span")[i]
        .get_text()
        .split(" : ")[0]: record_etc[0]
        .find_all("span")[i]
        .get_text()
        .split(" : ")[1]
        for i in range(0, len(record_etc[0].find_all("span")))
    }
    # etc={i.split(" : ")[0]:i.split(" : ")[1] for i in record_etc[0].get_text().split("\n") if len(i)!=0 }
    record.update(etc)
    return record


def away_batter(tables, team):
    temp1 = pd.read_html(str(tables[4]))[0].dropna()
    temp1 = temp1.rename(columns={"Unnamed: 1": "포지션"})
    del temp1["Unnamed: 0"]
    temp2 = pd.read_html(str(tables[5]))[0][:-1]
    temp3 = pd.read_html(str(tables[6]))[0][:-1]
    away = pd.concat([temp1, temp2, temp3], axis=1)
    away["팀"] = team[0]
    away = away.fillna(0)
    return away


def home_batter(tables, team):
    temp1 = pd.read_html(str(tables[7]))[0].dropna()
    temp1 = temp1.rename(columns={"Unnamed: 1": "포지션"})
    del temp1["Unnamed: 0"]
    temp2 = pd.read_html(str(tables[8]))[0][:-1]
    temp3 = pd.read_html(str(tables[9]))[0][:-1]
    home = pd.concat([temp1, temp2, temp3], axis=1)
    home["팀"] = team[1]
    home = home.fillna(0)
    return home


def away_pitcher(tables, team):
    away = pd.read_html(str(tables[10]))[0][:-1]
    away["팀"] = team[0]
    away = away.fillna(0)
    return away


def home_pitcher(tables, team):
    home = pd.read_html(str(tables[11]))[0][:-1]
    home["팀"] = team[1]
    home = home.fillna(0)
    return home
